;; From emacs master -Q

;; adjust paths
(add-to-list 'load-path "c:/Projects/elpa/packages/uniquify-files")
(add-to-list 'load-path "c:/Projects/elpa/packages/path-iterator")
(add-to-list 'load-path "c:/Projects/emacs_stephe.main/projects") ;; for emacs-project.el, uniquify-test-project.el

(find-file "c:/Projects/elpa/packages/uniquify-files/uniquify-files-resources/foo-file3.texts2")

(require 'uniquify-test-project)

;; adding user choice of completion style for list collections
(defcustom project-file-completion-styles '(substring)
  "Setting of `completion-styles' for project-find-file.
Note that a completion table function can specify styles,
overriding this.")

(defun project--completing-read-strict (prompt
                                        collection &optional predicate
                                        hist default inherit-input-method)
  ;; Tried both expanding the default before showing the prompt, and
  ;; removing it when it has no matches.  Neither seems natural
  ;; enough.  Removal is confusing; early expansion makes the prompt
  ;; too long.
  (let* (;; When COLLECTION (a completion table) is a function that
	 ;; returns category styles, we set
	 ;; completion-category-overrides to reflect the table. If the
	 ;; completion table returns metadata category, we assume it
	 ;; will be project-file.
	 (table-styles (cdr (assq 'styles (completion-metadata "" collection nil))))
	 (completion-category-overrides
	  (list (list 'project-file (cons 'styles
					  (or table-styles
					      project-file-completion-styles)))))

	 ;; If the completion table is a list, or a function that does
	 ;; not return styles metadata, we set completion-styles to
	 ;; reflect the user choice.
	 (completion-styles (if table-styles nil project-file-completion-styles))

	 (common-parent-directory
	  (let ((common-prefix (try-completion "" collection)))
	    (if (> (length common-prefix) 0)
		(file-name-directory common-prefix))))

	 (new-collection
	  (cond
	   ((or table-styles (not (memq 'file-root-rel completion-styles)))
	    collection)

	   (common-parent-directory
	    (apply-partially 'fc-root-rel-completion-table-list
			     collection common-parent-directory))

	   (t collection)))

	 (new-prompt (if default
                         (format "%s (default %s): " prompt default)
                       (format "%s: " prompt)))
         (res (completing-read new-prompt
                               new-collection predicate t
                               nil hist default inherit-input-method)))
    (if (and (equal res default)
             (not (test-completion res new-collection predicate)))
        (completing-read (format "%s: " prompt)
                         new-collection predicate t res hist nil
                         inherit-input-method)
      res)))

(with-current-buffer "foo-file3.texts2"
  (let ((uniquify-test-project-completion-style 'uniquify-file)
	(project-file-completion-styles '(substring)))
    (message "navigate to uniquify-test/Alice/alice-1/bar-file1.text, using uniquified filenames")
    (sit-for 1.0)
    (project-find-file)
    ))

(with-current-buffer "foo-file3.texts2"
  (let ((uniquify-test-project-completion-style 'default)
	(project-file-completion-styles '(uniquify-file)))
    (message "navigate to uniquify-test/Alice/alice-1/bar-file1.text, using uniquified filenames")
    (sit-for 1.0)
    (project-find-file)
    ))

(with-current-buffer "foo-file3.texts2"
  (let ((uniquify-test-project-completion-style 'file-root-rel)
	(project-file-completion-styles '(substring)))
    (message "navigate to uniquify-test/Alice/alice-2/foo-file1.text, using relative filenames")
    (sit-for 1.0)
    (project-find-file)
    ))

(with-current-buffer "foo-file3.texts2"
  (let ((uniquify-test-project-completion-style 'default)
	(project-file-completion-styles '(file-root-rel)))
    (message "navigate to uniquify-test/Alice/alice-1/bar-file1.text, using relative filenames")
    (sit-for 1.0)
    (project-find-file)
    ))

(with-current-buffer "foo-file3.texts2"
  (let ((uniquify-test-project-completion-style 'default)
	(project-file-completion-styles '(substring)))
    (message "navigate to uniquify-test/Bob/alice-3/foo-file4.text, using absolute filenames")
    (sit-for 1.0)
    (project-find-file)
    ))
